__title__ = 'pyrecard'
__description__ = 'A Python lib for brazilian Wirecard payment gateway.'
__url__ = 'https://github.com/DiegoMagg/pyrecard'
__version__ = '1.0.0'
__author__ = 'Diego Magalhães'
__author_email__ = 'dmlmagal@gmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright (c) 2020 Diego Magalhães'
